from email.header import Header
from email.mime.text import MIMEText


class MailBuilder:
    def __init__(self) -> None:
        pass

    def content(self, content):
        self._content = content
        return self

    def frm(self, name, usn):
        self._from = '%s <%s>' % (
            Header(name, 'utf-8').encode(), usn)
        return self

    def to(self, addr):
        self._to = addr
        return self

    def subject(self, subject):
        self._subject = Header(subject, 'utf-8').encode()
        return self

    def build(self):
        message = MIMEText(self._content, 'html', 'utf-8')
        message['From'] = self._from
        message['To'] = self._to
        message['Subject'] = self._subject
        return message



import argparse
from lspsp.executable import Executable
from lspsp.lspmon import Lspmon


class MonitorExecutable(Executable):
    def parse(self):
        parser = argparse.ArgumentParser(
            description='LSPSP Monitor Executable.')
        cmd_parser = parser.add_subparsers(dest='cmd', help="Command Options")
        cmd_parser.add_parser(
            'run', help='Run Monitor Service.')
        cmd_parser.add_parser(
            'test', help='Run Notification Test.')
        self._parser = parser
        self._args = parser.parse_args()

    def run(self):
        lspmon = Lspmon(1)
        lspmon.launch()

    def test(self):
        lspmon = Lspmon(0)
        lspmon.launch()

    def execute(self):
        if self._args.cmd == 'run':
            self.run()
        elif self._args.cmd == 'test':
            self.test()
        else:
            self._parser.print_help()



import json
from apscheduler.schedulers.blocking import BlockingScheduler
from lspsp.lspnotify import LspNotify

import logging
from lspsp.lspsp import Lspsp


class Lspmon:
    _interval = 60
    _current = {}

    def __init__(self, mode) -> None:
        f = open('config.json', 'r')
        text = f.read()
        f.close()
        data = json.loads(text)
        self._notify_config = data['notify_config']
        self._interval = data['interval']
        self._api = Lspsp(data['loginuser'])
        logging.basicConfig(
            level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self._logger = logging.getLogger(__name__)
        self._mode = mode
        self._notifier = LspNotify(mode)

    def diff(self, data):
        diff = {}
        for key in data.keys():
            if self._current.get(key) == None:
                diff[key] = data[key]
        return diff

    def notify(self, diff):
        self._logger.info("New goods available found: %s", diff)
        for key in self._notify_config.keys():
            if self._notify_config[key]['enabled']:
                notify_method = getattr(self._notifier, key)
                notify_method(self._notify_config[key], diff)

    def job(self):
        data = self._api.list()
        diff = self.diff(data)
        self._current = data
        if diff:
            self.notify(diff)

    def launch(self):
        if self._mode == 0:
            self.notify({})
        elif self._mode == 1:
            scheduler = BlockingScheduler(timezone='Asia/Shanghai')
            scheduler.add_job(self.job, 'interval',
                              seconds=self._interval, id='check_capicity')
            scheduler.start()



from abc import abstractmethod


class Executable:
    @abstractmethod
    def parse(self):
        pass

    @abstractmethod
    def execute(self):
        pass

    def __init__(self, argv):
        self._argv = argv
        self.parse()
        self.execute()


from setuptools import setup, find_packages

setup(
    name="lspsp",
    version="1.0.2-2",
    license='GPL',
    description='LSPSP.ME Monitor',
    long_description='LSPSP.ME Monitor Service',
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[
        'APScheduler==3.10.1',
        'beautifulsoup4==4.12.2',
        'Jinja2==3.1.1',
        'Requests==2.30.0',
        'setuptools==57.4.0',
    ],

    scripts=[],
    entry_points={
        'console_scripts': [
            'lspsp = lspsp.main:main'
        ]
    },
    data_files=[
        ('etc/lspsp', ['config.json.template'])
    ]
)



from email.header import Header
from email.mime.text import MIMEText
import logging
import smtplib
from jinja2 import Environment, FileSystemLoader
import os

from lspsp.mailbuilder import MailBuilder


class LspNotify:
    def __init__(self, mode) -> None:
        self._logger = logging.getLogger(__name__)
        self._mode = mode

    def get_subject(self, content):
        if self._mode == 0:
            return '收件收发配置测试'
        elif self._mode == 1:
            return '发现LSPSP.ME新库存%d个' % (len(content))
        else:
            return '未知类型的邮件'

    def mail(self, config, content):
        root = os.path.dirname(__file__)
        path = os.path.join(root, 'resource')
        env = Environment(loader=FileSystemLoader(path))
        template_name = ''
        if self._mode == 0:
            template_name = 'test'
        else:
            template_name = 'mail'
        template = env.get_template(template_name + '.html')
        html_content = str(template.render(content=content))

        smtp = None
        if config['ssl']:
            smtp = smtplib.SMTP_SSL(config['host'], config['port'])
        else:
            smtp = smtplib.SMTP(config['host'], config['port'])
        smtp.login(config['usn'], config['pwd'])

        for item in config['recv']:
            builder = MailBuilder().content(html_content).frm(config['name'], config['usn']).to(
                item).subject(self.get_subject(content))
            message = builder.build()

            smtp.sendmail(config['usn'], item, message.as_string())
            self._logger.info("Email sended to: %s", item)



import json
from bs4 import BeautifulSoup
import requests

LSP_HOST = 'https://www.lspsp.me'


class Lspsp:
    _loginuser = None

    def __init__(self, loginuser) -> None:
        self._loginuser = loginuser

    def api(self, uri, method, data):
        url = LSP_HOST + uri
        cookies = {
            "loginUser": self._loginuser
        }
        response = requests.request(
            method=method, url=url, data=data, cookies=cookies).text
        return response

    def list(self):
        ret = {}
        html = self.api('/bonus', 'GET', '')
        bs = BeautifulSoup(html, 'html.parser')
        content = bs.select_one('.main-area #content')
        if content == None:
            return ret
        for item in content.select('.lspfree'):
            btn = item.select_one('button')
            disabled = btn.attrs.get('disabled')
            if disabled == None:
                ret[btn.attrs['data-goid']] = {
                    "name": item.select_one('.title').text,
                    "info": item.select_one('.info ul li').text,
                }
        return ret
